% function FS=subsetFDFST(lf,Np)
% DESCRIPTION: Computes the subset Fast Discrete Focal Stack transform
% INPUT: lf, lightfield with Nxy x Nxy microlenses, Nuv x Nuv
% pixels behind each microlens and nbands per pixel (nbands=3 for color images) 
% stored in a [Nxy,Nxy,Nuv,Nuv,nbands] array . 
% lf can be obtained from an image using the read_lightfield3 function.
% Np, number of planes, it has to obey the following constraints
% Np has to be odd
% Np-1 has to divide (Nxy-Nxyodd +2*floor((Nuv+2)/4))*(Nuv-Nuvodd)
% Nxyodd=1, Nuvodd =1 if the number is odd , zero otherwise
% Input Np is increased until it verifies the constraint.
% OUTPUT: FS, focal stack  from the lightfield computed with the subset Fast Discrete
% Focal Stack Transform (DFST) stored in a [Nxy,Nxy,nbands,Np] 


function FS=subsetFDFST(lf,Np)
np=(Np-1)/2;
if np~=floor(np)
    np=ceil(np);
    Np=2*np+1;
    fprintf('Warning: Np has to be odd, new value for Np=%d\n',Np);
end

[Nxy,Nxy,Nuv,Nuv,nbands]=size(lf);

nuv=floor(Nuv/2);
nxy=floor(Nxy/2);

Nxyeven=(Nxy/2==nxy);
Nuveven=(Nuv/2==nuv);

Nuvex=Nuv+Nuveven;

addfilesdown=ceil(nuv/2);
Nxyex=Nxy+2*addfilesdown+Nxyeven;
nxyex=nxy+addfilesdown;

npext=2*nxyex*nuv;
Npext=2*npext+1;

mwarning=false;
while (npext/np)~=floor(npext/np)
    mwarning=true;
    np=np+1;
end

c=npext/np;
Np=2*np+1;

if mwarning
    fprintf('Warning: Non integer quotient, new value for Np =%d\n',Np);
end


FS=zeros([Nxy,Nxy,3,Np],'single');


prez=exp(pi*i*[1:2*np].^2*(-c/(np*Nxyex)));
conjprez=conj(prez(1:np));
shiftedzarr=[1 prez fliplr(prez)];
fftshiftedzarr=fft(shiftedzarr);  
weights=[fliplr(conjprez) 1 conjprez];

weights_final=zeros([Npext,1]);
for k=-npext:npext
    weights_final(k+npext+1)=exp(2*pi*i*k/(Nxyex*np));
end




zpimdata=zeros([Nxyex,Nxyex]);
beginzpim=addfilesdown+1+Nxyeven;
endzpim=Nxyex-addfilesdown;
zplensdata=zeros([Nuvex,Nuvex]);

for k=1:nbands
    fprintf('Generating band %d\n',k);

    S1=zeros([Nxyex,Nxyex,Nuv,Nuv]);
    fprintf('Generating S1... ');

    for u=-nuv:nuv-Nuveven
        for v=-nuv:nuv-Nuveven
            zpimdata(beginzpim:endzpim,beginzpim:endzpim)=reshape(lf(:,:,u+nuv+1,v+nuv+1,k),[Nxy,Nxy]);
            S1(:,:,u+nuv+1,v+nuv+1)=fftshift(fft2(ifftshift(zpimdata)));
        end
    end 
    
    fprintf('Done\n');
    S2=repmat(complex(0,0),[Nxyex,Nxyex,Np,c]);

    
   fprintf('Generating S2... ');
    for a=-nxyex:nxyex
        A=a+nxyex+1;
        for b=-nxyex:nxyex        
            B=b+nxyex+1;
            for u=-nuv:nuv-Nuveven
                U=u+nuv+1;
                for v=-nuv:nuv-Nuveven
                    V=v+nuv+1;
                    kindex=a*u+b*v;
                    qk=floor(kindex/c);
                    mk=mod(kindex,c);
                    S2(A,B,qk+np+1,mk+1)=S2(A,B,qk+np+1,mk+1)+S1(A,B,U,V);
                end
            end
        end
    end
    fprintf('Done\n'); 
    
    clear S1;
    S3=zeros(Nxyex,Nxyex,Np,c); 
    fprintf('Generating S3... ');
    for qk=-np:np
        for mk=0:c-1
            S3(:,:,qk+np+1,mk+1)=fftshift(ifft2(ifftshift(reshape(S2(:,:,qk+np+1,mk+1),[Nxyex,Nxyex]))));
        end
    end
    fprintf('Done\n'); 
    clear S2;
        
    fprintf('Generating S4... ');
    shiftedyarr=zeros([1,2*Np-1]);
    S4=zeros(Nxyex,Nxyex,Np,c);
    for mk=0:c-1
        for a=-nxyex:nxyex 
            for b=-nxyex:nxyex
                    cc=reshape(S3(a+nxyex+1,b+nxyex+1,:,mk+1),[1,Np]).*weights;
                    shiftedyarr(1:np+1)=cc(np+1:Np);
                    shiftedyarr(3*np+2:2*Np-1)=cc(1:np);
                    tmp0=ifft(fft(shiftedyarr).*fftshiftedzarr);
                    tmp1=[tmp0(3*np+2:2*Np-1) tmp0(1:1+np)];
                    S4(a+nxyex+1,b+nxyex+1,:,mk+1)=(tmp1.*weights);
           end  
        end
    end
    clear S3
    fprintf('Done\n'); 
    fprintf('Generating S5... ');
    S5=zeros(Nxyex,Nxyex,Np);
    for p=-np:np
        for mk=0:c-1
            kindex=p*mk;
            S5(:,:,p+np+1)=S5(:,:,p+np+1)+S4(:,:,p+np+1,mk+1)*exp(2*pi*i*kindex/(Nxyex*np));
        end
    end
    clear S4          
       
    fprintf('Done\n');
    FS(:,:,k,:)=single(squeeze(real((S5(beginzpim:endzpim,beginzpim:endzpim,:)))/((Nuvex-Nuveven)*(Nuvex-Nuveven))));
    clear S5
   
    fprintf('band %d Done\n',k);
end    


return;